local NUM_TRADING_SLOTS = 7
local interfaceItem = {}
local interfaceIndex = 0

local addonName, TF = ...

TF.signature = "|c8788EEff" .. addonName .. ":|r "

local TradeFill = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0")
local Loc = LibStub("AceLocale-3.0"):GetLocale(addonName)

local Minimap = LibStub("LibDBIcon-1.0", true)

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame)
local TradeButtonClear = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")

TF.options = {
    name = addonName,
    handler = TradeFill,
    type = "group",
    childGroups = "tab",
    args = {
        menu = {
            type = "group",
            name = Loc["TAB_ITEMS"],
            desc = Loc["TAB_ITEMS_DESC"],
            order = 1,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = Loc["TOGGLE_SHOW"],
                    desc = Loc["TOGGLE_SHOW_DESC"],
                    get = function(info) return TradeFill.db.profile.setting.show end,
                    set = function(info, value) TradeFill.db.profile.setting.show = value end,
                    order = 0,
                },
                mainTab = {
                    type = "group",
                    name = Loc["TAB_MAIN"],
                    desc = Loc["TAB_MAIN_DESC"],
                    order = 1,
                    args = {
                        toggleBoxC = {
                            type = 'toggle',
                            name = Loc["TOGGLE_AUTOFILL"],
                            desc = Loc["TOGGLE_AUTOFILL_DESC"],
                            get = function(info) return TradeFill.db.profile.setting.auto end,
                            set = function(info, value) return TradeFill:setAutofill(value) end,
                            order = 1,
                        },
                        toggleBoxB = {
                            type = 'toggle',
                            name = Loc["TOGGLE_FULL_STACKS"],
                            desc = Loc["TOGGLE_FULL_STACKS_DESC"],
                            get = function(info) return TradeFill.db.profile.setting.full end,
                            set = function(info, value) TradeFill.db.profile.setting.full = value end,
                            order = 2,
                        },
                    },
                },
                hideeTab = {
                    type = "group",
                    name = Loc["TAB_HIDE"],
                    desc = Loc["TAB_HIDE_DESC"],
                    order = 2,
                    args = {
                        toggleBoxC = {
                            type = 'toggle',
                            name = Loc["TOGGLE_MINIMAP"],
                            desc = Loc["TOGGLE_MINIMAP_DESC"],
                            get = function(info) return TradeFill.db.profile.minimap.hide end,
                            set = function() return TradeFill:setMinimapButton() end,
                            order = 1,
                        },
                        toggleBoxE = {
                            type = 'toggle',
                            name = Loc["TOGGLE_BUTTON"],
                            desc = Loc["TOGGLE_BUTTON_DESC"],
                            get = function(info) return TradeFill.db.profile.setting.button end,
                            set = function(info, value) TradeFill:enableDisableButton(value) end,
                            order = 2,
                        },
                        toggleBoxF = {
                            type = 'toggle',
                            name = Loc["TOGGLE_MESSAGES"],
                            desc = Loc["TOGGLE_MESSAGES_DESC"],
                            get = function(info) return TradeFill.db.profile.setting.message end,
                            set = function(info, value) TradeFill.db.profile.setting.message = value end,
                            order = 3,
                        },
                    },
                },
            },
        },
        stacks = {
            type = "group",
            name = Loc["TAB_STACKS"],
            desc = Loc["TAB_STACKS_DESC"],
            order = 2,
            args = {
            },
        },
        group = {
            type = "group",
            name = Loc["TAB_CONDITION"],
            desc = Loc["TAB_CONDITION_DESC"],
            order = 3,
            args = {
                group2 = {
                    type = "group",
                    name = Loc["TREE_CONDITION"],
                    desc = string.format(Loc["TREE_CONDITION_DESC"], getMaxPlayerLevel()),
                    order = 1,
                    args = {
                        toggleBoxA = {
                            type = 'toggle',
                            name = Loc["TOGGLE_STRANGER"],
                            desc = Loc["TOGGLE_STRANGER_DESC"],
                            get = function() return TradeFill.db.profile.setting.solo end,
                            set = function(info, value) TradeFill.db.profile.setting.solo = value end,
                            order = 1,
                        },
                        toggleBoxB = {
                            type = 'toggle',
                            name = Loc["TOGGLE_PARTY"],
                            desc = Loc["TOGGLE_PARTY_DESC"],
                            get = function() return TradeFill.db.profile.setting.party end,
                            set = function(info, value) TradeFill.db.profile.setting.party = value end,
                            order = 2,
                        },
                        toggleBoxC = {
                            type = 'toggle',
                            name = Loc["TOGGLE_RAID"],
                            desc = Loc["TOGGLE_RAID_DESC"],
                            get = function() return TradeFill.db.profile.setting.raid end,
                            set = function(info, value) TradeFill.db.profile.setting.raid = value end,
                            order = 3,
                        },
                        separatorA = {
                            type = "description",
                            name = " ",
                            order = 4,
                        },
                        toggleBoxD = {
                            type = 'toggle',
                            name = Loc["TOGGLE_GUILD"],
                            desc = Loc["TOGGLE_GUILD_DESC"],
                            get = function() return TradeFill.db.profile.setting.guild end,
                            set = function(info, value) TradeFill.db.profile.setting.guild = value end,
                            order = 5,
                        },
                        toggleBoxE = {
                            type = 'toggle',
                            name = Loc["TOGGLE_REQUIRED"],
                            desc = Loc["TOGGLE_REQUIRED_DESC"],
                            get = function() return TradeFill.db.profile.setting.require end,
                            set = function(info, value) TradeFill.db.profile.setting.require = value end,
                            order = 6,
                        },
                        toggleBoxF = {
                            type = 'toggle',
                            name = function() return getMaxPlayerLevel() end,
                            desc = Loc["TOGGLE_LEVEL_DESC"],
                            get = function() return TradeFill.db.profile.setting.level end,
                            set = function(info, value) TradeFill.db.profile.setting.level = value end,
                            order = 7,
                        },
                        separatorB = {
                            type = "description",
                            name = " ",
                            order = 8,
                        },
                        toggleBoxG = {
                            type = 'toggle',
                            name = Loc["TOGGLE_MASTER"],
                            desc = Loc["TOGGLE_MASTER_DESC"],
                            get = function() return TradeFill.db.profile.setting.master end,
                            set = function(info, value) TradeFill.db.profile.setting.master = value end,
                            order = 9,
                        },
                    },
                },
                player = {
                    type = "group",
                    name = Loc["TREE_PLAYERS"],
                    desc = Loc["TREE_PLAYERS_DESC"],
                    order = 2,
                    args = {
                        inputA = {
                            type = 'input',
                            name = Loc["INPUT_TEXTAREA_PLAYERS"],
                            desc = Loc["INPUT_TEXTAREA_PLAYERS_DESC"],
                            get = function() return TradeFill.db.profile.setting.players end,
                            set = function(info, value) TradeFill.db.profile.setting.players = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n]*$",
                            usage = Loc["INPUT_TEXTAREA_PLAYERS_USAGE"],
                            multiline = 15,
                            order = 1,
                        },
                    },
                },
                guild = {
                    type = "group",
                    name = Loc["TREE_GUILDS"],
                    desc = Loc["TREE_GUILDS_DESC"],
                    order = 3,
                    args = {
                        inputA = {
                            type = 'input',
                            name = Loc["INPUT_TEXTAREA_GUILDS"],
                            desc = Loc["INPUT_TEXTAREA_GUILDS_DESC"],
                            get = function() return TradeFill.db.profile.setting.guilds end,
                            set = function(info, value) TradeFill.db.profile.setting.guilds = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n ]*$",
                            usage = Loc["INPUT_TEXTAREA_GUILDS_USAGE"],
                            multiline = 15,
                            order = 1,
                        },
                    },
                },
            },
        },
        logbook = {
            type = "group",
            name = Loc["TAB_HISTORY"],
            desc = Loc["TAB_HISTORY_DESC"],
            order = 4,
            args = {
                all = {
                    type = "execute",
                    name = Loc["BUTTON_DELETE_ALL"],
                    desc = Loc["BUTTON_DELETE_ALL_DESC"],
                    func = function() TradeFill:DeleteHistoryAll() end,
                    confirm = true,
                    width = 0.75,
                }
            },
        },
    },
}

function TradeFill:OnInitialize()
    for index, class in pairs(classes) do
        TF.options.args.stacks.args[class] = {
            type = "group",
            name = setClass(class),
            order = index,
            args = {
            },
        }
    end

    LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, TF.options)
    local AceConfigDialog = LibStub("AceConfigDialog-3.0")
    AceConfigDialog:AddToBlizOptions(addonName, addonName)

    local defaultsItems = {
        profile = {
            item = {
                ["*"] = {
                    id = 0,
                    name = "",
                    link = "",
                    quality = 0,
                    level = 0,
                    stack = 0,
                    texture = 0,
                },
            },
            stacks = {
                ["*"] = {
                    ["1"] = 0,
                    ["2"] = 0,
                    ["3"] = 0,
                    ["4"] = 0,
                    ["5"] = 0,
                    ["6"] = 0,
                },
            },
            setting = {
                solo = false,
                party = false,
                raid = false,
                auto = true,
                show = true,
                full = true,
                button = false,
                message = false,
                require = false,
                level = false,
                master = false,
                guild = false,
                guilds = "",
                players = "",
            },
            minimap = {
                hide = false,
            },
        },
    }

    local tradeFillHistory = {
        profile = {
            ["*"] = {
                player = "",
                tradePlayer = {
                    money = "",
                    ["*"] = {
                        ["*"] = {
                            id = 0,
                            name = "",
                            texture = 0,
                            quantity = 0,
                            link = ""
                        },
                    },
                },
                target = "",
                tradeTarget = {
                    money = "",
                    ["*"] = {
                        ["*"] = {
                            id = 0,
                            name = "",
                            texture = 0,
                            quantity = 0,
                            link = ""
                        },
                    },
                },
            },
        },
    }

    self.db = LibStub("AceDB-3.0"):New("TradeFillDB", defaultsItems)
    self.history = LibStub("AceDB-3.0"):New("TradeFillHistoryDB", tradeFillHistory)

    TF.data = self.db.profile

    TradeFill:InitializeInterfaceOption()
    TradeFill:InitializeInterfaceOptionStack()
    TradeFill:InitializeHistory()

    Minimap:InitializeMinimapButton(self)

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)

    CreateTradeButton()

    TradeButtonClear:SetText("Clear")
    TradeButtonClear:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    TradeButtonClear:SetSize(75, 22)
    TradeButtonClear:SetScript("OnClick", function() clearTade() end)

    self:RegisterChatCommand("tradefill", "SlashCommand")
    self:RegisterChatCommand("tradefillon", "On")
    self:RegisterChatCommand("tradefilloff", "Off")

    DEFAULT_CHAT_FRAME:AddMessage(string.format(TF.signature .. Loc["VERSION"], C_AddOns.GetAddOnMetadata(addonName, "Version")))

    if TradeFill.db.profile.item["1"].id > 0 and TradeFill.db.profile.item["1"].name == "" then
        TradeFill:SetSelectItem("1", TradeFill.db.profile.item["1"].id)
    end

    if TradeFill.db.profile.item["2"].id > 0 and TradeFill.db.profile.item["2"].name == "" then
        TradeFill:SetSelectItem("2", TradeFill.db.profile.item["2"].id)
    end

    if TradeFill.db.profile.item["a"].id > 0 or TradeFill.db.profile.item["b"].id > 0 then
        local itemA = false
        local itemB = false

        if TradeFill.db.profile.setting.standby then
            TradeFill.db.profile.setting.auto = TradeFill.db.profile.setting.standby
            TradeFill.db.profile.setting.standby = nil
        end

        if TradeFill.db.profile.item["a"].id > 0 then
            TradeFill:SetSelectItem("1", TradeFill.db.profile.item["a"].id)
            itemA = true
        end

        if TradeFill.db.profile.item["b"].id > 0 then
            TradeFill:SetSelectItem("2", TradeFill.db.profile.item["b"].id)
            itemB = true
        end

        for key, class in pairs(classes) do
            if TradeFill.db.profile.stacks[class]["a"] ~= nil then
                if TradeFill.db.profile.stacks[class]["a"] > 0 then
                    TradeFill.db.profile.stacks[class][tostring(1)] = TradeFill.db.profile.stacks[class]["a"]
                    TradeFill.db.profile.stacks[class]["a"] = nil
                end
            end

            if TradeFill.db.profile.stacks[class]["b"] ~= nil then
                if TradeFill.db.profile.stacks[class]["b"] > 0 then
                    TradeFill.db.profile.stacks[class][tostring(2)] = TradeFill.db.profile.stacks[class]["b"]
                    TradeFill.db.profile.stacks[class]["b"] = nil
                end
            end
        end

        if itemA then
            if TradeFill.db.profile.item["1"].id then
                if TradeFill.db.profile.item["1"].name then
                    if TradeFill.db.profile.item["1"].link then
                        if TradeFill.db.profile.item["1"].quality then
                            if TradeFill.db.profile.item["1"].level then
                                if TradeFill.db.profile.item["1"].stack then
                                    if TradeFill.db.profile.item["1"].texture then
                                        TradeFill.db.profile.item["a"] = nil
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end

        if itemB then
            if TradeFill.db.profile.item["2"].id then
                if TradeFill.db.profile.item["2"].name then
                    if TradeFill.db.profile.item["2"].link then
                        if TradeFill.db.profile.item["2"].quality then
                            if TradeFill.db.profile.item["2"].level then
                                if TradeFill.db.profile.item["2"].stack then
                                    if TradeFill.db.profile.item["2"].texture then
                                        TradeFill.db.profile.item["b"] = nil
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    self:RegisterEvent("TRADE_SHOW")
end

function TradeFill:InitializeInterfaceOption()
    local optionIndex = 1

    for i = 1, MAX_TRADABLE_ITEMS do
        TF.options.args.menu.args[tostring(optionIndex)] = {
            type = "description",
            name = " ",
            order = optionIndex,
        }

        TF.options.args.menu.args[tostring(optionIndex + 1)] = {
            type = "select",
            width = 2,
            name = function() return TradeFill:GetName(tostring(i)) end,
            desc = Loc["OPTIONS_ITEM_DESC"],
            values = function() return TradeFill:SetOptionItem(tostring(i)) end,
            get = function() return TradeFill:GetInputItem(tostring(i)) end,
            set = function(info, value) TradeFill:SetSelectItem(tostring(i), tonumber(value)) end,
            order = optionIndex + 1,
        }
        optionIndex = optionIndex + 2
    end
    TF.options.args.menu.args[tostring(optionIndex)] = {
        type = "description",
        name = " ",
        order = optionIndex,
    }
end

function TradeFill:InitializeInterfaceOptionStack()
    local classIndex = 1

    for _, class in pairs(classes) do
        for i = 1, MAX_TRADABLE_ITEMS do
            TF.options.args.stacks.args[class].args[tostring(classIndex)] = {
                type = "description",
                fontSize = "medium",
                name = " ",
                order = classIndex,
            }

            TF.options.args.stacks.args[class].args[tostring(classIndex + 1)] = {
                type = "select",
                width = 1.5,
                name = function() return TradeFill:GetName(tostring(i)) end,
                desc = Loc["TREE_STACKS_SELECT_DESC"],
                values = function() return TradeFill:SetOptionStack() end,
                get = function() return tostring(TradeFill.db.profile.stacks[class][tostring(i)]) end,
                set = function(info, value) TradeFill.db.profile.stacks[class][tostring(i)] = tonumber(value) end,
                order = classIndex + 1,
            }
            classIndex = classIndex + 2
        end
        classIndex = 1
    end
end

local TradeButtons = {}

function CreateTradeButton()
    local buttonOrder = { 28, 62, 96, 130, 164, 198 }

    for index, name in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
        local button = CreateFrame("Button", name, OptionFrame)
        button:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 34, buttonOrder[index])
        button:SetSize(32, 32)

        button.label = button:CreateFontString(nil, "OVERLAY", "NumberFontNormal")
        button.label:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 34, buttonOrder[index])

        TradeButtons[name] = button
    end
end

function ButtonInitialize()
    for index, name in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
    local item = TradeFill.db.profile.item[tostring(index)]

        if item.id > 0 then
            TradeButtons[name]:Show()
            TradeButtons[name].label:SetText(TF.stackCount[index].stack)
            TradeButtons[name]:SetNormalTexture(GetItemIcon(item.id))
            TradeButtons[name]:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
            TradeButtons[name]:SetScript("OnClick", function()
                fillTradeWindow(TradeFill.db.profile.item[tostring(index)], tradePosition())
            end)
        else
            TradeButtons[name]:Hide()
        end
    end
end

function ButtonUpdate()
    ButtonInitialize()

    for clearSlot = 1, MAX_TRADABLE_ITEMS do
        TF.stackCount[clearSlot].trade = 0
    end

    for slot = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(slot) ~= nil then
            local name, _, quantity, _, _, _ = GetTradePlayerItemInfo(slot)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradePlayerItemLink(slot))

            if name ~= nil then

                for key = 1, MAX_TRADABLE_ITEMS do
                    if TF.stackCount[key].id == id then
                        TF.stackCount[key].trade = TF.stackCount[key].trade + quantity
                        for index, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
                            if key == index then
                                TradeButtons[button].label:SetText(tostring(TF.stackCount[key].stack - TF.stackCount[key].trade))
                            end
                        end
                    end
                end
            end
        end
    end
end

function TradeFill:setAutofill()
    if TradeFill.db.profile.setting.auto then
        self.db.profile.setting.auto = false
        Minimap:setButtonOff()
    else
        self.db.profile.setting.auto = true
        Minimap:setButtonOn()
    end
end

function TradeFill:setMinimapButton()
    if TradeFill.db.profile.minimap.hide then
        self.db.profile.minimap.hide = false
        Minimap:Show("minimap")
    else
        self.db.profile.minimap.hide = true
        Minimap:Hide("minimap")
    end
end

function TradeFill:enableDisableClassSetting(value, option)
    local key = tostring(tonumber(option * 2))

    if value == 0 then
        for _, class in pairs(classes) do
            TF.options.args.stacks.args[class].args[key].disabled = true
        end
    else
        for _, class in pairs(classes) do
            TF.options.args.stacks.args[class].args[key].disabled = false
        end
    end
end

function TradeFill:enableDisableButton(value)
    if value then
        for _, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
            TradeButtons[button]:Hide()
            TradeButtonClear:Hide()
        end
    else
        for _, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
            TradeButtons[button]:Show()
            TradeButtonClear:Show()
        end
    end

    TradeFill.db.profile.setting.button = value
end

function TradeFill:SetOptionStack()
    local selectOption = {}

    for i = 0, MAX_TRADABLE_ITEMS do
        selectOption[tostring(i)] = i
    end

    return selectOption
end

function TradeFill:SetOptionItem(option)
    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                interfaceIndex = interfaceIndex + 1
                interfaceItem[interfaceIndex] = bagItem
            end
        end
    end

    local selectOption = {}

    selectOption["0"] = Loc["OPTION_NONE"]

    for i, slot in pairs(interfaceItem) do

        local _, _, _, _, _, _, _, itemStackCount = GetItemInfo(slot.itemID)

        if TradeFill.db.profile.setting.show == true then
            if slot.isBound == false and slot.stackCount == itemStackCount then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if TradeFill.db.profile.item[option].id > 0 then
            local item = TradeFill.db.profile.item[option].id
            local itemLink = TradeFill.db.profile.item[option].link

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end
    end
    interfaceItem = {}
    interfaceIndex = 0
    return selectOption
end

function TradeFill:GetInputItem(option)
    local value = TradeFill.db.profile.item[option].id
    TradeFill:enableDisableClassSetting(value, option)
    return tostring(value)
end

function TradeFill:SetSelectItem(option, value)
    local itemName, itemLink, itemQuality, _, itemMinLevel, _, _,
    itemStackCount, _, itemTexture = GetItemInfo(value)

    local set = self.db.profile.item[option]
    set.id = value
    set.name = itemName
    set.link = itemLink
    set.quality = itemQuality
    set.level = itemMinLevel
    set.stack = itemStackCount
    set.texture = itemTexture
end

function TradeFill:GetName(option)
    local level = TradeFill.db.profile.setting.require
    local item = self.db.profile.item[option]

    if item.id > 0 then
        if item.link == nil then
            ReloadUI()
        end

        if TradeFill.db.profile.item["1"].id > 0 and TradeFill.db.profile.item["1"].name == "" then
            ReloadUI()
        end

        if TradeFill.db.profile.item["2"].id > 0 and TradeFill.db.profile.item["2"].name == "" then
            ReloadUI()
        end

        if level == true then
            return "|T" .. GetItemIcon(item.id) .. ":24[:24[:2:3|t  " .. item.link:gsub("[%[%]]", "") .. setColor(Loc["ITEM_NAME"] .. " ".. item.level,  "FFD100")
        else
            return "|T" .. GetItemIcon(item.id) .. ":24[:24[:2:3|t  " .. item.link:gsub("[%[%]]", "")
        end
    else
        return "|T" .. GetItemIcon(1217) .. ":24[:24[:2:3|t  " .. Loc["ITEM_UNKNOWN"]
    end
end

function TradeFill:SlashCommand()
    InterfaceOptionsFrame_OpenToCategory(TradeFill)
end

function TradeFill:On()
    TradeFill.db.profile.setting.auto = true

    Minimap:setButtonOn()
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["ADDON_ON"])
end

function TradeFill:Off()
    TradeFill.db.profile.setting.auto = false

    Minimap:setButtonOff()
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["ADDON_OFF"])
end